/*
Example program that integrates mkHttpd.
This runs only with a thread-enabled Tcl/Tk.
Make sure TCL_LIBRARY is set to where init.tcl is.
(because Tcl_Init doesn't work in a thread)
*/

#include <stdio.h>
#include <string.h>
#include <tcl.h>

char pcText[1000] = "";
int  iCount = 0;

int getData( ClientData pC, Tcl_Interp *pI, int objc, Tcl_Obj *CONST objv[] )
{
  if( objc != 2 )
  {
    Tcl_SetStringObj( Tcl_GetObjResult( pI ), "Wrong # args: Should be getData value", -1 );
  }
  else if( ! strcmp( Tcl_GetStringFromObj( objv[1], NULL ), "text" ) )
  {
    Tcl_SetObjResult( pI, Tcl_NewStringObj( pcText, -1 ) );
  }
  else if( ! strcmp( Tcl_GetStringFromObj( objv[1], NULL ), "count" ) )
  {
    Tcl_SetObjResult( pI, Tcl_NewIntObj( iCount ) );
  }
  else 
  {
    Tcl_SetStringObj( Tcl_GetObjResult( pI ), "Bad value for getData: Must be text or count", -1 );
  }

  return TCL_OK;
}

Tcl_ThreadCreateType startServer( ClientData pC )
{
  Tcl_Interp *pI;

  char pcScript[] = 
    "package require mkHttpd;"
    "member Session:getData { what } { getData $what };"
    "Httpd www -port 1088 -default ctest.tsp;"
    "vwait forever";

  if( ! ( pI = Tcl_CreateInterp() ) )
  {
    fprintf( stderr, "\nError: Tcl_CreateInterp failed\n" );
    Tcl_ExitThread(1);
  }

  if( Tcl_Init( pI ) != TCL_OK )
  {
    fprintf( stderr, "\nError: Tcl_Init failed: %s\n", Tcl_GetStringResult( pI ) );
    Tcl_ExitThread(2);
  }

  Tcl_CreateObjCommand( pI, "getData", getData, NULL, NULL );

  if( Tcl_Eval( pI, pcScript ) != TCL_OK )
  { 
    fprintf( stderr, "\nError: Tcl_Eval failed: %s\n", Tcl_GetStringResult( pI ) );
    Tcl_ExitThread(2);
  }

  Tcl_ExitThread(0);
  return 0;
}

int startThread( char *pcArgv0 )
{
  Tcl_ThreadId hThread;

  Tcl_FindExecutable( pcArgv0 );
  Tcl_CreateThread( &hThread, startServer, NULL, TCL_THREAD_STACK_DEFAULT, TCL_THREAD_NOFLAGS );

  return 0;
}

int main( int argc, char **argv )
{
  startThread( argv[0] );

  while( 1 )
  {
    printf( "Enter some text: " );
    scanf( "%s", pcText );  
    printf( "Count=%d, Text='%s'\n\n", ++iCount, pcText );
  }

  return 0;
}

